searchNodes=[{"doc":"RedactEx provides protocols and derivation utilities to work with sensitive data that should be redacted in logs and external facing tools RedactEx.Redactor module gives you automagical generation of redacting functions under a desired module namespace RedactEx.Redactable is the protocol for which implementation and derivation should be created to ensure the best possible practices in your codebase Protect your data RedactEx usual protection consists in two or more steps: Generate your redactor functions You can generate functions to redact your specific data using RedactEx.Redactor module. iex&gt; defmodule MyApp.Redacting do ...&gt; @moduledoc false ...&gt; use RedactEx.Redactor , redactors : [ ...&gt; { &quot;redact_three&quot; , length : 3 , algorithm : :simple } , ...&gt; { &quot;redact&quot; , lengths : 1 .. 3 , algorithm : :simple } ...&gt; ] ...&gt; end This will generate optimized functions and/or functions adopting the standards and algorithms defined in this library. In particular: redact_three/1 will be created with an optimized match for strings of length 3 and a non optimized fallback function for strings with generic length redact/1 will be created with three optimized match for strings of lengths 1, 2 and 3 and a non optimized fallback function for strings with generic length Protect your structs The recommended way of protecting your structs is 1. Derive or implement RedactEx.Redactable You can use the @derive macro from the RedactEx.Redactable protocol, configured based on your needs and optionally using functions from a module generated with RedactEx.Redactor helpers, or manually define an implementation of choice, e.g. iex&gt; defmodule MyRedactorModule do ...&gt; def redact_function_one ( _ ) , do : &quot;(redacted1)&quot; ...&gt; def redact_function_two ( _ ) , do : &quot;(redacted2)&quot; ...&gt; end ...&gt; defmodule MyApp.RedactStruct do ...&gt; @derive { RedactEx.Redactable , ...&gt; fields : [ ...&gt; myfield1 : { MyRedactorModule , :redact_function_one } , ...&gt; myfield2 : { MyRedactorModule , :redact_function_two } , ...&gt; ] } ...&gt; defstruct [ :myfield1 , :myfield2 ] ...&gt; end or iex&gt; defmodule MyApp.OtherRedactStruct do ...&gt; defstruct [ :myfield1 , :myfield2 ] ...&gt; end ...&gt; defimpl RedactEx.Redactable , for : MyApp.OtherRedactStruct do ...&gt; def redact ( _value ) , do : % MyApp.OtherRedactStruct { myfield1 : &quot;(redacted)&quot; , myfield2 : &quot;(redacted)&quot; } ...&gt; end 2. Use inspect protocol at your advantage No strategy can eliminate the risk of leaking sensitive data with code only. Our suggested approach is to derive also the inspect protocol and use it for your struct whenever you log or send data to external systems, e.g. iex&gt; defimpl Inspect , ...&gt; for : MyApp.OtherRedactStruct do ...&gt; alias MyApp.OtherRedactStruct ...&gt; alias RedactEx.Redactable ...&gt; ...&gt; @spec inspect ( OtherRedactStruct . t ( ) , Inspect.Opts . t ( ) ) :: Inspect.Algebra . t ( ) ...&gt; def inspect ( input , opts ) do ...&gt; input ...&gt; |&gt; Redactable . redact ( ) ...&gt; |&gt; Inspect.Any . inspect ( opts ) ...&gt; end ...&gt; end Another strategy could be to directly call RedactEx.Redactable.redact/1 on structs when logging, but using inspect seems to us more natural and idiomatic. Another strategy could be to redact / inspect your data directly in a custom Logger backend, or whatever the system used for exporting potentially sensitive data.","ref":"RedactEx.html","title":"RedactEx","type":"module"},{"doc":"Dispatcher of supported algorithms","ref":"RedactEx.Algorithms.html","title":"RedactEx.Algorithms","type":"module"},{"doc":"","ref":"RedactEx.Algorithms.html#algorithm_simple/0","title":"RedactEx.Algorithms.algorithm_simple/0","type":"function"},{"doc":"","ref":"RedactEx.Algorithms.html#base_algorithms/0","title":"RedactEx.Algorithms.base_algorithms/0","type":"function"},{"doc":"Get the AST of a redacting function from given module and configuration","ref":"RedactEx.Algorithms.html#generate_ast/2","title":"RedactEx.Algorithms.generate_ast/2","type":"function"},{"doc":"Validate an algorithm at configuration level","ref":"RedactEx.Algorithms.html#valid_algorithm?/1","title":"RedactEx.Algorithms.valid_algorithm?/1","type":"function"},{"doc":"Behaviour to implement for an algorithm to be configured as a redacting algorithm when using RedactEx","ref":"RedactEx.Algorithms.Algorithm.html","title":"RedactEx.Algorithms.Algorithm","type":"behaviour"},{"doc":"Returns the AST of the implementing algorithm, given whatever supported configuration","ref":"RedactEx.Algorithms.Algorithm.html#c:generate_ast/1","title":"RedactEx.Algorithms.Algorithm.generate_ast/1","type":"callback"},{"doc":"Check that this behaviour is implemented in given module","ref":"RedactEx.Algorithms.Algorithm.html#implemented_by?/1","title":"RedactEx.Algorithms.Algorithm.implemented_by?/1","type":"function"},{"doc":"Parses additional configuration at compile time for given algorithm. Configuration will be kept in Context's extra field Shall return any configuration (eventually) readable by generate_ast and generate_fallback_ast The function shall raise in case of errors","ref":"RedactEx.Algorithms.Algorithm.html#c:parse_extra_configuration!/1","title":"RedactEx.Algorithms.Algorithm.parse_extra_configuration!/1","type":"callback"},{"doc":"","ref":"RedactEx.Algorithms.Algorithm.html#t:t/0","title":"RedactEx.Algorithms.Algorithm.t/0","type":"type"},{"doc":"Simple redacting algorithm that will keep the initial and last parts of a string and substitutes the remaining part with given character # e.g. value_to_redact -&gt; va * * * * * * * * * * * ct","ref":"RedactEx.Algorithms.Center.html","title":"RedactEx.Algorithms.Center","type":"module"},{"doc":"Simple redacting algorithm that will keep the initial part of a string and substitutes the remaining part with given character # e.g. value_to_redact -&gt; valu * * * * * * * * * * * *","ref":"RedactEx.Algorithms.Simple.html","title":"RedactEx.Algorithms.Simple","type":"module"},{"doc":"Configuration helpers for handling RedactEx configuration","ref":"RedactEx.Configuration.html","title":"RedactEx.Configuration","type":"module"},{"doc":"Return configurations grouped by name, so we can generate matching and catchall in a good order when needed","ref":"RedactEx.Configuration.html#parse/3","title":"RedactEx.Configuration.parse/3","type":"function"},{"doc":"Context for generating a redacting function","ref":"RedactEx.Configuration.Context.html","title":"RedactEx.Configuration.Context","type":"module"},{"doc":"For fixed-length redacted strings, returns the length of the plaintext to be kept of the original information","ref":"RedactEx.Configuration.Context.html#get_plaintext_length_redacted_length/3","title":"RedactEx.Configuration.Context.get_plaintext_length_redacted_length/3","type":"function"},{"doc":"Gets the actual redacted part of the string given configuration","ref":"RedactEx.Configuration.Context.html#get_redactor_string/2","title":"RedactEx.Configuration.Context.get_redactor_string/2","type":"function"},{"doc":"","ref":"RedactEx.Configuration.Context.html#t:t/0","title":"RedactEx.Configuration.Context.t/0","type":"type"},{"doc":"Redactable Protocol for defining a redact-able item, e.g. an item whose internal elements could be redacted It shall return a redact-ed item of the same type as the input item You can derive Redactable protocol for a struct by using defmodule MyApp.RedactStruct do @derive { RedactEx.Redactable , fields : [ myfield1 : { MyModule , :redact_function_one } , myfield2 : { MyModule , :redact_function_two } , ] } defstruct [ :myfield1 , :myfield2 ] end options :fields possible values of fields are a keyword list of configurations, where configuration can be {module, function} , e.g. fields : [ myfield1 : { MyModule , :redact_function_one } , myfield2 : { MyModule , :redact_function_two } , ] In this case redacting will be applied to myfield1 and myfield2 using module.function as configured :redact , e.g. fields : [ field : :redact ] In this case it is expected for value in field to have an Impl for RedactEx.Redactable , and RedactEx.Redactable.redact/1 will be called :drop , e.g. fields : [ field : :drop ] In this case :field key is entirely dropped. NOTE that this also means struct information will be lost, and the value will become a map e.g. suppose you define a module like # ./test/support/derive/redact_struct.ex defmodule MyApp.RedactStruct do @moduledoc false # For usage in RedactEx.Redactable doctests @derive { RedactEx.Redactable , fields : [ myfield : { String , :reverse } ] } defstruct [ :myfield ] end then you can expect it to have derived the RedactEx.Redactable protocol ` ` ` iex&gt; % MyApp.RedactStruct { myfield : &quot;reverseme&quot; } |&gt; RedactEx.Redactable . redact ( ) |&gt; Map . get ( :myfield ) &quot;emesrever&quot; ` ` ` Best Practices Good practices could be to implement the Inspect protocol for sensitive data and use the derived redact capabilities of the inspected module inside its implementation. You can both derive single fields in your struct, or define whatever implementation fits best for your structs as defimpl RedactEx.Redactable, for: MyStruct Then you can enforce inspect ing structures in your logs or export functions. Unluckily this is not a simple practice to enforce in an automatic flavour.","ref":"RedactEx.Redactable.html","title":"RedactEx.Redactable","type":"protocol"},{"doc":"Redacts value hiding sensitive information","ref":"RedactEx.Redactable.html#redact/1","title":"RedactEx.Redactable.redact/1","type":"function"},{"doc":"","ref":"RedactEx.Redactable.html#t:any_container/0","title":"RedactEx.Redactable.any_container/0","type":"type"},{"doc":"","ref":"RedactEx.Redactable.html#t:binary_container/0","title":"RedactEx.Redactable.binary_container/0","type":"type"},{"doc":"","ref":"RedactEx.Redactable.html#t:container/0","title":"RedactEx.Redactable.container/0","type":"type"},{"doc":"","ref":"RedactEx.Redactable.html#t:t/0","title":"RedactEx.Redactable.t/0","type":"type"},{"doc":"RedactEx Utility that defines a set of functions to fast-redact data where possible, e.g. when you already know the length of the string you're gonna have as validated input Redacting is unidirectional and is meant to help the safety of logs and exported data Note that right now this function matches on binary data, so you probably want to consider it if UTF-stuff is heavily involved when defining lengths. For example, in extreme cases, considering that a UTF-8 string may have a string length of 5 (e.g. héllo ) but a byte size of 6 (as by documentation ) iex&gt; string = &quot;héllo&quot; &quot;héllo&quot; iex&gt; String . length ( string ) 5 iex&gt; byte_size ( string ) 7 So you may consider it when evaluating for which kind of data you're planning to use byte-based helpers Algorithms Optimized algorithms for strings with expected length(s) Those algorithms will match on the byte_size of the string Algorithm Description :simple keeps the first keep % of the string and sets the rest to redactor , according to redacted_size :center splits keep % of the string between head and tail, and sets the middle to redactor , according to redacted_size Configuration Options: redactors a list of redacting rules. Each element can be: - a tuple in the form { redactor_function_name , configuration } - a tuple in the form { [ redactor_function_names ] , configuration } - a single redactor_function_name ` redactor_function_name ` can be an atom or string value in all cases Common configuration Configuration of a redactor is a keyword list containing redactor string character used to redact the hidden part of the. Defaults to * redacted_size integer | :auto length of the resulting string that will be set as redacted. Defaults to auto , which will set it to ` expected_string_length - keep ` algorithm atom | module algorithm used to redact the string. Defaults to simple if a module is given , it must implement the ` RedactEx.Algorithms.Algorithm ` behaviour Supported atoms are - ` :simple ` ( alias for ` RedactEx.Algorithms.Simple ` ) - ` :center ` ( alias for ` RedactEx.Algorithms.Center ` ) keep integer quote of the string that will be kept. Defaults to 25% length integer or :* length of the string to be considered. :* stands for the fallback function configuration. Is overridden by key :lengths if present lengths [integer or :* ] | min..max lengths of the strings to be considered. :* stands for the fallback function configuration. A function for each specific length will be generated except list(atom()) list of environments for which this configuration will have effect fallback_value string fixed value for redacting non-string values Example iex&gt; defmodule MyApp.Redacting do ...&gt; @moduledoc false ...&gt; use RedactEx.Redactor , redactors : [ ...&gt; { &quot;redact_three&quot; , length : 3 , algorithm : :simple } , ...&gt; { &quot;redact&quot; , lengths : 1 .. 3 , algorithm : :simple } ...&gt; ] ...&gt; end iex&gt; MyApp.Redacting . redact ( &quot;test&quot; ) ...&gt; &quot;t***&quot; This will expose the following functions : * ` MyApp.Redacting . redact ` with fast matches on input of length 1 , 2 , 3 * ` MyApp.Redacting . redact_three ` with fast matches on input of length 3 * both ` redact / 1 ` and ` redact_three / 1 ` will have a slower fallback function for input of different length Custom algorithms You can define custom algorithms to define your own implementations. Take a look at RedactEx.Algorithms.Algorithm behaviour.","ref":"RedactEx.Redactor.html","title":"RedactEx.Redactor","type":"module"},{"doc":"redact_ex Utilities to redact sensitive data in Elixir projects","ref":"readme.html","title":"README","type":"extras"},{"doc":"By redacting we consider here the practice of hiding part of a potentially sensitive information, while keeping a small part of it to allow understanding that it may be a valid one. For example, you may want to redact a telephone number in your logs, but keep the initial digits (e.g. the prefix) because that information is relevant to your understanding of the system, and at the same moment it does not expose any sensitive information.","ref":"readme.html#redacting","title":"README - Redacting","type":"extras"},{"doc":"","ref":"readme.html#configuration","title":"README - Configuration","type":"extras"},{"doc":"Full documentation is available on hexdocs See RedactEx for general information about this library and its intended use See RedactEx.Redactor for generating fast redactors for strings See RedactEx.Redactable protocol for deriving redactable structs","ref":"readme.html#usage","title":"README - Usage","type":"extras"},{"doc":"The recommended way to work is to use asdf language runtime versions manager. You will find a .tool-versions.recommended file that - hopefully - is up to date with the versions used in CI. Please be sure that your test pass with those versions before pushing to save yourself the wait for automatic checks. # Use project&#39;s defaults cp .tool-versions.recommended .tool-versions","ref":"readme.html#contributing","title":"README - Contributing","type":"extras"},{"doc":"[ ] Expand explanation of fallback custom implementations of algorithms [ ] fix dialyzer :(","ref":"readme.html#todo","title":"README - TODO","type":"extras"},{"doc":"Other ways of redacting (e.g. first X letters and last Y exposed, random redactors, ecc) maybe-email redactor define a default strict implementation for redact in the Any implementation? some other default magic configuration in derive, e.g. a default module+function? better handling of fallbacks for derived structs, and macro-generated helpers to &quot;try_redact&quot; structs that may be nil or other","ref":"readme.html#open-points","title":"README - Open Points","type":"extras"}]